@@ -154,11 +154,32 @@ sub clone {
     return $cloned;
 }
 
+sub clone_list {
+    my $class = shift;
+    my @clones = map { $_->clone } @_;
+    @clones;    
+}
+
+sub detach {
+    my $self = shift;
+    my $parent = $self->parent;
+    #$self->{node}->unbindNode();
+    my $doc = XML::LibXML->createDocument;
+    $doc->adoptNode($self->{node});
+    $doc->setDocumentElement($self->{node});    
+    $parent;
+}
+
 sub delete {
     my $self = shift;
     $self->{node}->unbindNode();
 }
 
+sub delete_content {
+    my ($self) = @_;
+    $self->{node}->removeChildNodes;
+}
+
 sub getFirstChild {
     my $self = shift;
     __PACKAGE__->new($self->{node}->getFirstChild);
@@ -173,6 +194,59 @@ sub childNodes {
     wantarray ? @nodes : \@nodes;
 }
 
+sub content_list {
+    my ($self) = @_;
+    my @nodes = $self->childNodes;
+    @nodes;
+}
+
+sub replace_with {
+    my $self = shift;
+    
+    my $node   = $self->{node}; 
+    my $doc    = $node->ownerDocument;
+    my $parent = $node->parentNode;        
+    my @nodes  = map { ref $_ ? $_->{node} : $doc->createTextNode($_) } @_;
+    
+    $node->addSibling($_)
+        foreach @nodes; 
+
+    my $newdoc = XML::LibXML->createDocument;
+    $newdoc->adoptNode($self->{node});
+    $newdoc->setDocumentElement($self->{node});
+    
+    $self;    
+}
+
+sub push_content {
+    my $self = shift;
+    
+    foreach (@_) {
+        ref $_ ? $self->{node}->appendChild($_->{node})
+               : $self->{node}->appendText($_);
+    }
+    
+    $self;    
+}
+
+sub unshift_content {
+    my $self = shift;
+    
+    return $self->push_content(@_) 
+        unless $self->{node}->hasChildNodes;
+
+    my $node = $self->{node}; 
+    my $doc = $node->ownerDocument;
+    my $first_child = $node->firstChild;
+    
+    foreach (@_) {
+        ref $_ ? $node->insertBefore($_->{node}, $first_child)
+               : $node->insertBefore($doc->createTextNode($_), $first_child);
+    }
+    
+    $self;    
+}
+
 sub left {
     my $self = shift;
 
@@ -267,17 +341,34 @@ sub matches {
 
 sub parent {
     my $self = shift;
+    
     if (@_) {    
+        
+        # unset
+        unless (defined $_[0]) {
+            $self->{node}->unbindNode;   
+            return;            
+        }
+        
         # set        
-        if (defined $_[0]) {
-            Carp::croak "an element can't be made its own parent"
-                if ref $_[0]->{node}->isEqual($self->{node});    # sanity
-                
-            $_[0]->{node}->appendChild($self->{node});                        
+        Carp::croak "an element can't be made its own parent"
+            if ref $_[0]->{node}->isSameNode($self->{node});    # sanity
+            
+        my $parent = $_[0]->{node};
+        
+        if ($_[0]->{node}->isa('XML::LibXML::Document')) {
+
+            if ($parent->hasChildNodes) {
+                $parent->lastChild->addSibling($self->{node});
+            }
+            else {
+                $parent->adoptNode($self->{node});
+                $parent->setDocumentElement($self->{node});                    
+            }                
+            
         }
         else {
-            # unset
-            $self->{node}->unbindNode;    
+            $parent->appendChild($self->{node});                
         }
                 
     }
@@ -289,6 +380,30 @@ sub parent {
 
 }
 
+sub postinsert {
+    my $self   = shift;    
+    my @nodes  = map { $_->{node} } @_;
+    my $parent = $self->{node}->parentNode;
+    
+    $parent->insertAfter($_, $self->{node})
+        foreach reverse @nodes;
+    
+    $self;    
+}
+
+sub preinsert {
+    my $self   = shift;    
+    my @nodes  = map { $_->{node} } @_;
+    my $parent = $self->{node}->parentNode;
+    
+    $parent->insertBefore($_, $self->{node})
+        foreach @nodes;
+    
+    $self;    
+}
+
+
+
 1;
 
 __END__
